# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:52:30+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerIDModel(BaseModel):
    identifier: Optional[Any] = None



