# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:18:10+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerStatusModel(BaseModel):
    result: Optional[str] = None
    connected: Optional[bool] = None



