# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-02-20T07:32:48+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class TautulliInfoModel(BaseModel):
    tautulli_install_type: Optional[str] = None
    tautulli_version: Optional[str] = None
    tautulli_branch: Optional[str] = None
    tautulli_commit: Optional[str] = None
    tautulli_platform: Optional[str] = None
    tautulli_platform_release: Optional[str] = None
    tautulli_platform_version: Optional[str] = None
    tautulli_platform_linux_distro: Optional[str] = None
    tautulli_platform_device_name: Optional[str] = None
    tautulli_python_version: Optional[str] = None
