# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:25:38+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class ChildrenListItemModel(BaseModel):
    actors: Optional[List] = None
    added_at: Optional[str] = None
    art: Optional[str] = None
    audience_rating: Optional[str] = None
    audience_rating_image: Optional[str] = None
    banner: Optional[str] = None
    content_rating: Optional[str] = None
    directors: Optional[List] = None
    duration: Optional[str] = None
    full_title: Optional[str] = None
    collections: Optional[List] = None
    genres: Optional[List] = None
    grandparent_rating_key: Optional[str] = None
    grandparent_thumb: Optional[str] = None
    grandparent_title: Optional[str] = None
    guid: Optional[str] = None
    labels: Optional[List] = None
    last_viewed_at: Optional[str] = None
    library_name: Optional[str] = None
    media_index: Optional[str] = None
    media_type: Optional[str] = None
    original_title: Optional[str] = None
    originally_available_at: Optional[str] = None
    parent_media_index: Optional[str] = None
    parent_rating_key: Optional[str] = None
    parent_thumb: Optional[str] = None
    parent_title: Optional[str] = None
    rating: Optional[str] = None
    rating_image: Optional[str] = None
    rating_key: Optional[str] = None
    section_id: Optional[str] = None
    sort_title: Optional[str] = None
    studio: Optional[str] = None
    summary: Optional[str] = None
    tagline: Optional[str] = None
    thumb: Optional[str] = None
    title: Optional[str] = None
    updated_at: Optional[str] = None
    user_rating: Optional[str] = None
    writers: Optional[List] = None
    year: Optional[str] = None
    collections: Optional[List] = None


class ChildrenMetadataModel(BaseModel):
    children_count: Optional[int] = None
    children_type: Optional[str] = None
    title: Optional[str] = None
    children_list: Optional[List[ChildrenListItemModel]] = None



