# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:05:45+00:00

from __future__ import annotations

from typing import Any, List, Optional, Union

from pydantic import BaseModel


class DatumModel(BaseModel):
    row_id: Optional[int] = None
    user_id: Optional[int] = None
    username: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    plays: Optional[int] = None
    duration: Optional[int] = None
    last_seen: Optional[int] = None
    last_played: Optional[str] = None
    history_row_id: Optional[int] = None
    ip_address: Optional[str] = None
    platform: Optional[str] = None
    player: Optional[str] = None
    rating_key: Optional[int] = None
    media_type: Optional[str] = None
    thumb: Optional[str] = None
    parent_title: Optional[str] = None
    year: Optional[Any] = None
    media_index: Optional[Union[int, str]] = None
    parent_media_index: Optional[Union[int, str]] = None
    live: Optional[int] = None
    originally_available_at: Optional[str] = None
    guid: Optional[str] = None
    transcode_decision: Optional[str] = None
    do_notify: Optional[int] = None
    keep_history: Optional[int] = None
    allow_guest: Optional[int] = None
    is_active: Optional[int] = None
    title: Optional[str] = None
    email: Optional[str] = None


class UsersTableModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None



