# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:22:54+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class ServersInfoEntryModel(BaseModel):
    name: Optional[str] = None
    machine_identifier: Optional[str] = None
    host: Optional[str] = None
    port: Optional[str] = None
    version: Optional[str] = None
