# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:34:08+00:00

from __future__ import annotations

from typing import Any, Optional, Union

from pydantic import BaseModel


class GeoIPLookupModel(BaseModel):
    city: Optional[str] = None
    code: Optional[str] = None
    continent: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[Union[float, None]] = None
    longitude: Optional[Union[float, None]] = None
    postal_code: Optional[str] = None
    region: Optional[str] = None
    timezone: Optional[str] = None
    accuracy: Optional[Any] = None



