# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:59:53+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class UserModel(BaseModel):
    row_id: Optional[int] = None
    user_id: Optional[int] = None
    username: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    email: Optional[str] = None
    is_active: Optional[int] = None
    is_admin: Optional[int] = None
    is_home_user: Optional[int] = None
    is_allow_sync: Optional[int] = None
    is_restricted: Optional[int] = None
    do_notify: Optional[int] = None
    keep_history: Optional[int] = None
    allow_guest: Optional[int] = None
    deleted_user: Optional[int] = None
    shared_libraries: Optional[List] = None



