# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:38:48+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class LibraryUserStatsModel(BaseModel):
    friendly_name: Optional[str] = None
    user_id: Optional[int] = None
    user_thumb: Optional[str] = None
    username: Optional[str] = None
    total_plays: Optional[int] = None



