# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:54:30+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class NotifierParameterModel(BaseModel):
    name: Optional[str] = None
    type: Optional[str] = None
    value: Optional[str] = None
