# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T20:59:57+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class SeriesModel(BaseModel):
    name: Optional[str] = None
    data: Optional[List] = None


class PlaysOrStreamTypesByModel(BaseModel):
    categories: Optional[List[str]] = None
    series: Optional[List[SeriesModel]] = None



