# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:23:14+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    timestamp: Optional[int] = None
    art_level: Optional[int] = None
    complete: Optional[int] = None
    custom_fields: str
    exists: Optional[bool] = None
    export_id: Optional[int] = None
    exported_items: Optional[int] = None
    file_format: Optional[str] = None
    file_size: Optional[int] = None
    filename: Optional[Any] = None
    individual_files: Optional[int] = None
    media_info_level: Optional[int] = None
    media_type: Optional[str] = None
    media_type_title: Optional[str] = None
    metadata_level: Optional[int] = None
    rating_key: Optional[Any] = None
    section_id: Optional[int] = None
    thumb_level: Optional[int] = None
    title: Optional[str] = None
    total_items: Optional[int] = None
    user_id: Optional[Any] = None


class ExportsTableModel(BaseModel):
    draw: Optional[int] = None
    recordsTotal: Optional[int] = None
    recordsFiltered: Optional[int] = None
    data: Optional[List[DatumModel]] = None



