# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:05:19+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class UserWatchTimeStatsModel(BaseModel):
    query_days: Optional[int] = None
    total_plays: Optional[int] = None
    total_time: Optional[int] = None



