# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:26:04+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class UpdateCheckModel(BaseModel):
    update: Optional[bool] = None
    install_type: Optional[str] = None
