# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:57:17+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class PMSUpdateModel(BaseModel):
    update_available: Optional[bool] = None
    platform: Optional[str] = None
    release_date: Optional[int] = None
    version: Optional[str] = None
    requirements: Optional[str] = None
    extra_info: Optional[str] = None
    changelog_added: Optional[str] = None
    changelog_fixed: Optional[str] = None
    label: Optional[str] = None
    distro: Optional[str] = None
    distro_build: Optional[str] = None
    download_url: Optional[str] = None
