# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T01:28:24+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class DocsModel(BaseModel):
    add_newsletter_config: Optional[str] = None
    add_notifier_config: Optional[str] = None
    arnold: Optional[str] = None
    backup_config: Optional[str] = None
    backup_db: Optional[str] = None
    delete_all_library_history: Optional[str] = None
    delete_all_user_history: Optional[str] = None
    delete_cache: Optional[str] = None
    delete_export: Optional[str] = None
    delete_history: Optional[str] = None
    delete_hosted_images: Optional[str] = None
    delete_image_cache: Optional[str] = None
    delete_library: Optional[str] = None
    delete_login_log: Optional[str] = None
    delete_lookup_info: Optional[str] = None
    delete_media_info_cache: Optional[str] = None
    delete_mobile_device: Optional[str] = None
    delete_newsletter: Optional[str] = None
    delete_newsletter_log: Optional[str] = None
    delete_notification_log: Optional[str] = None
    delete_notifier: Optional[str] = None
    delete_recently_added: Optional[str] = None
    delete_synced_item: Optional[str] = None
    delete_temp_sessions: Optional[str] = None
    delete_user: Optional[str] = None
    docs: Optional[str] = None
    docs_md: Optional[str] = None
    download_config: Optional[str] = None
    download_database: Optional[str] = None
    download_export: Optional[str] = None
    download_log: Optional[str] = None
    download_plex_log: Optional[str] = None
    edit_library: Optional[str] = None
    edit_user: Optional[str] = None
    export_metadata: Optional[str] = None
    get_activity: Optional[str] = None
    get_apikey: Optional[str] = None
    get_collections_table: Optional[str] = None
    get_date_formats: Optional[str] = None
    get_export_fields: Optional[str] = None
    get_exports_table: Optional[str] = None
    get_geoip_lookup: Optional[str] = None
    get_history: Optional[str] = None
    get_home_stats: Optional[str] = None
    get_libraries: Optional[str] = None
    get_libraries_table: Optional[str] = None
    get_library: Optional[str] = None
    get_library_media_info: Optional[str] = None
    get_library_names: Optional[str] = None
    get_library_user_stats: Optional[str] = None
    get_library_watch_time_stats: Optional[str] = None
    get_logs: Optional[str] = None
    get_metadata: Optional[str] = None
    get_new_rating_keys: Optional[str] = None
    get_newsletter_config: Optional[str] = None
    get_newsletter_log: Optional[str] = None
    get_newsletters: Optional[str] = None
    get_notification_log: Optional[str] = None
    get_notifier_config: Optional[str] = None
    get_notifier_parameters: Optional[str] = None
    get_notifiers: Optional[str] = None
    get_old_rating_keys: Optional[str] = None
    get_playlists_table: Optional[str] = None
    get_plays_by_date: Optional[str] = None
    get_plays_by_dayofweek: Optional[str] = None
    get_plays_by_hourofday: Optional[str] = None
    get_plays_by_source_resolution: Optional[str] = None
    get_plays_by_stream_resolution: Optional[str] = None
    get_plays_by_stream_type: Optional[str] = None
    get_plays_by_top_10_platforms: Optional[str] = None
    get_plays_by_top_10_users: Optional[str] = None
    get_plays_per_month: Optional[str] = None
    get_plex_log: Optional[str] = None
    get_pms_update: Optional[str] = None
    get_recently_added: Optional[str] = None
    get_server_friendly_name: Optional[str] = None
    get_server_id: Optional[str] = None
    get_server_identity: Optional[str] = None
    get_server_info: Optional[str] = None
    get_server_list: Optional[str] = None
    get_server_pref: Optional[str] = None
    get_servers_info: Optional[str] = None
    get_settings: Optional[str] = None
    get_stream_data: Optional[str] = None
    get_stream_type_by_top_10_platforms: Optional[str] = None
    get_stream_type_by_top_10_users: Optional[str] = None
    get_synced_items: Optional[str] = None
    get_user: Optional[str] = None
    get_user_ips: Optional[str] = None
    get_user_logins: Optional[str] = None
    get_user_names: Optional[str] = None
    get_user_player_stats: Optional[str] = None
    get_user_watch_time_stats: Optional[str] = None
    get_users: Optional[str] = None
    get_users_table: Optional[str] = None
    get_whois_lookup: Optional[str] = None
    import_config: Optional[str] = None
    import_database: Optional[str] = None
    logout_user_session: Optional[str] = None
    notify: Optional[str] = None
    notify_newsletter: Optional[str] = None
    notify_recently_added: Optional[str] = None
    pms_image_proxy: Optional[str] = None
    refresh_libraries_list: Optional[str] = None
    refresh_users_list: Optional[str] = None
    register_device: Optional[str] = None
    restart: Optional[str] = None
    search: Optional[str] = None
    server_status: Optional[str] = None
    set_mobile_device_config: Optional[str] = None
    set_newsletter_config: Optional[str] = None
    set_notifier_config: Optional[str] = None
    sql: Optional[str] = None
    status: Optional[str] = None
    terminate_session: Optional[str] = None
    undelete_library: Optional[str] = None
    undelete_user: Optional[str] = None
    update: Optional[str] = None
    update_check: Optional[str] = None
    update_metadata_details: Optional[str] = None
