# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:38:24+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    video_resolution: Optional[str] = None
    audio_channels: Optional[str] = None
    container: Optional[str] = None
    thumb: Optional[str] = None
    media_index: Optional[str] = None
    title: Optional[str] = None
    bitrate: Optional[str] = None
    video_codec: Optional[str] = None
    section_type: Optional[str] = None
    section_id: Optional[int] = None
    added_at: Optional[str] = None
    sort_title: Optional[str] = None
    file_size: Optional[str] = None
    parent_rating_key: Optional[str] = None
    audio_codec: Optional[str] = None
    year: Optional[str] = None
    media_type: Optional[str] = None
    parent_media_index: Optional[str] = None
    grandparent_rating_key: Optional[str] = None
    rating_key: Optional[str] = None
    video_framerate: Optional[str] = None
    last_played: Optional[int] = None
    play_count: Optional[int] = None


class LibraryMediaInfoModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None
    last_refreshed: Optional[int] = None
    filtered_file_size: Optional[int] = None
    total_file_size: Optional[int] = None



