# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:20:41+00:00

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel


class MetadataFieldModel(BaseModel):
    field: Optional[str] = None
    level: Optional[int] = None


class MediaInfoFieldModel(BaseModel):
    field: Optional[str] = None
    level: Optional[int] = None


class ExportFieldsModel(BaseModel):
    metadata_fields: Optional[List[MetadataFieldModel]] = None
    media_info_fields: Optional[List[MediaInfoFieldModel]] = None
