# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:51:24+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class LibraryNameModel(BaseModel):
    section_id: Optional[int] = None
    section_name: Optional[str] = None
    section_type: Optional[str] = None
    agent: Optional[str] = None
