# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:03:28+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    browser: Optional[str] = None
    friendly_name: Optional[str] = None
    host: Optional[str] = None
    ip_address: Optional[str] = None
    os: Optional[str] = None
    timestamp: Optional[int] = None
    user: Optional[str] = None
    user_agent: Optional[str] = None
    user_group: Optional[str] = None
    user_id: Optional[int] = None


class UserLoginsModel(BaseModel):
    draw: Optional[int] = None
    recordsTotal: Optional[int] = None
    recordsFiltered: Optional[int] = None
    data: Optional[List[DatumModel]] = None



