# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T19:32:43+00:00

from __future__ import annotations

from typing import List, Optional, Any

from pydantic import BaseModel


class ItemUserStatModel(BaseModel):
    friendly_name: Optional[str] = None
    user_id: Optional[int] = None
    user_thumb: Optional[str] = None
    username: Optional[str] = None
    total_plays: Optional[int] = None
    total_time: Optional[int] = None



