# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:04:32+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class UserPlayerStatsModel(BaseModel):
    platform: Optional[str] = None
    platform_name: Optional[str] = None
    player_name: Optional[str] = None
    result_id: Optional[int] = None
    total_plays: Optional[int] = None



