# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:51:38+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class PlexLogModel(BaseModel):
    data: Optional[List] = None



