# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:02:13+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    friendly_name: Optional[str] = None
    guid: Optional[str] = None
    id: Optional[int] = None
    ip_address: Optional[str] = None
    last_played: Optional[str] = None
    last_seen: Optional[int] = None
    first_seen: Optional[int] = None
    live: Optional[int] = None
    media_index: Optional[int] = None
    media_type: Optional[str] = None
    originally_available_at: Optional[str] = None
    parent_media_index: Optional[int] = None
    parent_title: Optional[str] = None
    platform: Optional[str] = None
    play_count: Optional[int] = None
    player: Optional[str] = None
    rating_key: Optional[int] = None
    thumb: Optional[str] = None
    transcode_decision: Optional[str] = None
    user_id: Optional[int] = None
    year: Optional[int] = None


class UserIPsModel(BaseModel):
    draw: Optional[int] = None
    recordsTotal: Optional[int] = None
    recordsFiltered: Optional[int] = None
    data: Optional[List[DatumModel]] = None



