# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:37:01+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class LibraryModel(BaseModel):
    row_id: Optional[int] = None
    server_id: Optional[str] = None
    section_id: Optional[int] = None
    section_name: Optional[str] = None
    section_type: Optional[str] = None
    library_thumb: Optional[str] = None
    library_art: Optional[str] = None
    count: Optional[int] = None
    parent_count: Optional[Any] = None
    child_count: Optional[Any] = None
    is_active: Optional[int] = None
    do_notify: Optional[int] = None
    do_notify_created: Optional[int] = None
    keep_history: Optional[int] = None
    deleted_section: Optional[int] = None



