# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:51:11+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    addedAt: Optional[str] = None
    composite: Optional[str] = None
    duration: Optional[int] = None
    guid: Optional[str] = None
    leafCount: Optional[int] = None
    librarySectionID: Optional[str] = None
    playlistType: Optional[str] = None
    ratingKey: Optional[int] = None
    smart: Optional[bool] = None
    summary: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None
    updatedAt: Optional[str] = None
    userID: Optional[Any] = None


class PlaylistsTableModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None



