# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2024-03-13T23:55:21+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class RecentlyAddedItemModel(BaseModel):
    media_type: str
    section_id: str
    library_name: str
    rating_key: str
    parent_rating_key: str
    grandparent_rating_key: str
    title: str
    parent_title: str
    grandparent_title: str
    original_title: str
    sort_title: str
    media_index: str
    parent_media_index: str
    studio: str
    content_rating: str
    summary: str
    tagline: str
    rating: str
    rating_image: str
    audience_rating: str
    audience_rating_image: str
    user_rating: str
    duration: str
    year: str
    thumb: str
    parent_thumb: str
    grandparent_thumb: str
    art: str
    banner: str
    originally_available_at: str
    added_at: str
    updated_at: str
    last_viewed_at: str
    guid: str
    directors: List
    writers: List
    actors: List[str]
    genres: List[str]
    labels: List
    collections: List
    guids: List[str]
    full_title: str
    child_count: str


class RecentlyAddedModel(BaseModel):
    recently_added: Optional[List[RecentlyAddedItemModel]] = None



