# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:37:27+00:00

from __future__ import annotations

from typing import Any, List, Optional, Union

from pydantic import BaseModel


class DatumModel(BaseModel):
    row_id: Optional[int] = None
    server_id: Optional[str] = None
    section_id: Optional[int] = None
    section_name: Optional[str] = None
    section_type: Optional[str] = None
    count: Optional[int] = None
    parent_count: Optional[int] = None
    child_count: Optional[int] = None
    library_thumb: Optional[str] = None
    library_art: Optional[str] = None
    plays: Optional[int] = None
    duration: Optional[int] = None
    last_accessed: Optional[int] = None
    history_row_id: Optional[int] = None
    last_played: Optional[str] = None
    rating_key: Optional[int] = None
    media_type: Optional[str] = None
    thumb: Optional[str] = None
    parent_title: Optional[str] = None
    year: Optional[Union[int, str]] = None
    media_index: Optional[Union[int, str]] = None
    parent_media_index: Optional[Union[int, str]] = None
    content_rating: Optional[str] = None
    labels: Optional[List] = None
    live: Optional[int] = None
    originally_available_at: Optional[str] = None
    guid: Optional[str] = None
    do_notify: Optional[int] = None
    do_notify_created: Optional[int] = None
    keep_history: Optional[int] = None
    is_active: Optional[int] = None


class LibrariesTableModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None



