# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:34:08+00:00

from __future__ import annotations

from typing import Any, Optional, Union

from pydantic import BaseModel


class StatusModel(BaseModel):
    result: Optional[str] = None
    message: Any = None
