# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:58:36+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerIdentityModel(BaseModel):
    machine_identifier: Optional[str] = None
    version: Optional[str] = None
